/*
    connectDaily Web Calendar Software

    Copyright 2015, MH Software, Inc. All Rights Reserved
    5023 W 120th Ave #311, Broomfield CO 80020
    +1 303 438 9585

    Last Edit By: "$Author: gsexton $"
    Last Checkin: "$Date: 2018/03/02 18:54:30 $"
    Revision #  : "$Revision: 1.2 $"
*/

/*
    This file contains scripts used by the component feature screen.

*/

/**
*	This function unchecks any features that are contained within
*	this feature. E.G. if you check the unlimited resource feature,
*	ensure the basic feature set and the basic resource set are 
*	turned off.
*
*	@param parentItem The feature to check children of.
*/
function toggleChildren(parentItem) {
    require( [ "dojo/dom", "dojo/dom-attr" ], function(dom, domAttr) {
            var theParent = dom.byId('IDchk' + parentItem);
            var sFeatures = domAttr.get(theParent, "data-child-features");
            var a = sFeatures.split(" ");
            var bCheck = domAttr.get(theParent, "checked");
            if (bCheck) {
                domAttr.set(theParent, "data-selfref", "true");
            } else {
                domAttr.remove(theParent, "data-selfref");
            }
            for (var i = 0; i < a.length; i++) {
                var childNode = dom.byId("IDchk" + a[i]);
                if (childNode != null) {
                    var refCount = domAttr.get(childNode, "data-refcount");
                    if (refCount == null) {
                        refCount = 0;
                    } else {
                        refCount = parseInt(refCount);
                    }
                    refCount += bCheck ? 1 : -1;
                    domAttr.set(childNode, "data-refcount", refCount >= 0 ? refCount : 0);
                    if (refCount > 0) {
                        domAttr.set(childNode, "disabled", "");
                        domAttr.set(childNode, "checked", true);
                    } else {
                        domAttr.remove(childNode, "disabled");
                        if (domAttr.get(childNode, "data-selfref") == null) {
                            domAttr.set(childNode, "checked", false);
                        }
                    }
                }
            }
        });
    updateCost();
}

/**
 * This function is called in response to the toggling of a 
 * feature that's part of a feature pack. The big thing we need 
 * to do is look at the cost 
 */
function togglePackFeature(FeatureID) {
    require( [ "dojo/dom", "dojo/dom-attr" ], function(dom, domAttr) {
            var theFeature = dom.byId('IDchk' + FeatureID);
            var theParent = dom.byId('IDchk' + domAttr.get(theFeature, "data-set-tag"));
            var bCheck = domAttr.get(theFeature, "checked");
            if (bCheck) {
                domAttr.set(theFeature, "data-selfref", "true");
            } else {
                updateCost();
                domAttr.remove(theFeature, "data-selfref");
                return;
            }
            var sFeatures = domAttr.get(theParent, "data-child-features");
            var a = sFeatures.split(" ");
            var sum = 0;
            var parentCost = parseFloat(domAttr.get(theParent, "data-cost"));
            for (var i = 0; i < a.length; i++) {
                var childNode = dom.byId("IDchk" + a[i]);
                if (childNode != null && domAttr.get(childNode, "data-set-tag") != null) {
                    if (domAttr.get(childNode, "checked")) {
                        sum += parseFloat(domAttr.get(childNode, "data-cost"));

                        if (sum >= parentCost) {
                            domAttr.set(theParent, "checked", true);
                            domAttr.remove(theFeature, "data-selfref"); // We want to do this so that if pro pack is unchecked, this is unchecked.
                            toggleChildren(domAttr.get(theFeature, "data-set-tag"));
                            return;
                        }
                        updateCost();
                    }
                }
            }
        });
}

/**
 * Essentially, this function is going to iterate over the form 
 * elements and sum the checked ones. 
 */
function updateCost() {

    require( [ "dojo/dom", "dojo/dom-attr", "dojo/query" ], function(dom, domAttr, query) {
            var cost = 0;
            query("input[type=checkbox]:checked").forEach(
                function(node, index, arr) {
                    var thisCost = parseFloat(domAttr.get(node, "data-cost"));
                    var disabledStat = domAttr.get(node, "disabled");
                    console.log("updateCost - Examining: " + domAttr.get(node, "id"));
                    if (domAttr.get(node, "id") == "IDchkMULTIPLE_CALENDARS") {
                        var maxCost=parseFloat(domAttr.get(node,"data-maximum-cost")), 
                            iCount=parseInt(domAttr.get(node,"data-calendar-count"));
                        if (iCount > 0) {
                            var lineCost=thisCost * iCount;
                            if (lineCost>maxCost) {
                                lineCost=maxCost;
                            }
                            cost += lineCost;
                        }
                       
                    } else if (!disabledStat) {
                        cost += thisCost;
                    }
                });
            var discountNode=dom.byId('IDtxtNPDiscount');
            if (discountNode!=null) {
                var threshold=parseFloat(domAttr.get(discountNode,'data-np-threshold')),
                    discountRate=parseFloat(domAttr.get(discountNode,'data-np-discount')),
                        discount=discountRate*cost;
                if (cost>=threshold) {
                    cost-=discount;
                } else {
                    discount=0;
                }
                domAttr.set(discountNode,'value',discount.toFixed(2));
            }
            var costNode = dom.byId("IDtxtCost");
            domAttr.set(costNode, "value", cost.toFixed(2));
            var btn=dom.byId("IDbtnTrialCreate");
            if (btn!=null) {
                domAttr.remove(btn,"disabled")
            }
        });
}

/**
 * This function is our reset button handler. Essentially, we 
 * want to: 
 *  
 * 1) Make all the form elements enabled. 
 * 2) Call form.reset() 
 * 3) For each checked item that's not part of a set, call 
 * toggleChildren() 
 * 4) call updateCost();
 */
function handleReset() {
    require( [ "dojo/dom", "dojo/dom-attr", "dojo/query" ], function(dom, domAttr, query) {
            var cost = 0;
            query("input[type=checkbox]").forEach(
                function(node, index, arr) {
                    var sID = domAttr.get(node, "id");
                    if (sID == "IDchkMULTIPLE_CALENDARS") {
                        // This is a special case.
                        var txtBox = dom.byId("IDtxtAddlCalendars");
                        if (txtBox == null) {
                            domAttr.remove(node, "disabled");
                        }
                    } else {
                        domAttr.remove(node, "disabled");
                    }
                });
            var frm = dom.byId("IDfrmFeatures");
            frm.reset();
            query("input[type=checkbox]").forEach(
                function(node, index, arr) {
                    var sChildren = domAttr.get(node, "data-child-features");
                    if (sChildren != null) {
                        toggleChildren(domAttr.get(node, "id").substring(5));
                    }
                });

            updateCost();
            var btn=dom.byId("IDbtnTrialCreate");
            if (btn!=null) {
                domAttr.set(btn,"disabled",true)
            }
        
        });
}

function selectFeature(featureID) {
    require( [ "dojo/dom", "dojo/dom-attr" ], function(dom, domAttr) {
            var chk = dom.byId("IDchk" + featureID);
            domAttr.set(chk, "checked", true);
            toggleChildren(featureID);
            toggleDiv(featureID);
        });
}


